:: *****************************************************************************
::
::                          Illusonic Audio Processor
::
:: Wrapper for firmware update.
::
:: Version: 2.9.0
::
:: Copyright Illusonic GmbH, 2013-2016, Switzerland.  All rights reserved.
:: Illusonic proprietary and confidential.
:: *****************************************************************************

@echo off

:: *** config ******************************************************************

set IAP_TOOL=data\iap2_tool.exe
set IAP_TOOL_vA_1_0=data\iap2_tool_2.0.0.exe
set IAP_TOOL_vA_1_6=data\iap2_tool_2.6.0.exe
set IAP_TOOL_vA_1_7=data\iap2_tool_2.7.0.exe
set PIC_FIRMWARE=data\pic.hex
set DSP24_FIRMWARE=data\dsp_iap2+4.ldr
set DSP08_FIRMWARE=data\dsp_iap8.ldr
set DSP16_FIRMWARE=data\dsp_iap16.ldr
set DSP16D_FIRMWARE=data\dsp_iap16d.ldr
set NMR_FIRMWARE=data\nmr.bin
set HDMI_FIRMWARE_14=data\hdmi14.bin
set HDMI_FIRMWARE_20a=data\hdmi20a.bin
set HDMI_FIRMWARE_20b=data\hdmi20b.bin
set HDMI_FIRMWARE_20c=data\hdmi20c.bin
set HDMI_FIRMWARE_21a=data\hdmi21a.bin
set HDMI_FIRMWARE_21b=data\hdmi21b.bin
set HDMI_VERSION_14=0B-27-06
set HDMI_VERSION_20=2C-2B-11
set HDMI_VERSION_21=38-2E-29
set HDMI_VERSION_14_PREFIX=0B-
set HDMI_VERSION_20_PREFIX=2C-
set HDMI_VERSION_21_PREFIX=38-
set NMR_VERSION=1.16
set LIBDIR=%APPDATA%\Illusonic
set IAP_SETTINGS=%LIBDIR%\iap_settings.txt
set IAP_SETTINGS_TMP=%LIBDIR%\iap_settings.txt.tmp

goto entry_firmware_update


:: *** exit function ***********************************************************

:exit_action
    if %errorlevel% NEQ 0 (
        echo(
        echo Error ^(code %errorlevel%^)
        echo(
        echo Please check the instructions.
        echo For further assistance contact us at support@illusonic.com
    )
    echo(
    echo Press any key to exit...
    pause >nul
    exit
goto:eof


:: *** ask for user input ******************************************************

:ask_user_yes_no
    if "%RETRY_BANNER%" NEQ "" echo %RETRY_BANNER%
    :ask_user_yes_no_1
    set userinp=%~3
    set /p userinp=%RETRY_QUESTION%
    set userinp=%userinp:~0,1%
    if /i "%userinp%"=="%~1" (
        set ANSW=1
        goto ask_user_yes_no_2
    )
    if /i "%userinp%"=="%~2" (
        set ANSW=2
        goto ask_user_yes_no_2
    )
    goto ask_user_yes_no_1
    :ask_user_yes_no_2
goto:eof


:: *** run iap_tool, retry on failure ******************************************

:exec_iap_tool
    set RETRY=0
    if "%BANNER%" NEQ "" echo %BANNER%
    %IAP_TOOL% %*
    if %errorlevel% NEQ 0 (
        echo(
        set RETRY_BANNER=Warning: Process returned an error.
        set RETRY_QUESTION="Would you like to retry (R) or abort (a) ? "
        call :ask_user_yes_no R a R
        if !ANSW! EQU 1 (
            echo Retrying
            set RETRY=1
        ) else (
            echo Aborting
            call :exit_action
        )
    )
    if %RETRY% EQU 1 call :exec_iap_tool %*
goto:eof


:: *** get HDMI version ********************************************************

:exec_iap_tool_get_hdmi_version
    set RETRY=0
    set IAP_HDMI_VERSION=-1
    if "%BANNER%" NEQ "" echo %BANNER%
    for /F "delims=" %%i in ('%IAP_TOOL% version') do (
        set temp=%%i
        set temp=!temp:HDMI^:=!
        if %%i NEQ !temp! (
            set temp=!temp: =!
            set IAP_HDMI_VERSION=!temp!
        )
    )
    if !IAP_HDMI_VERSION! EQU -1 (
        set RETRY_BANNER=Warning: Process returned an error.
        set RETRY_QUESTION="Would you like to retry (R) or abort (a) ? "
        echo(
        call :ask_user_yes_no R a R
        if !ANSW! EQU 1 (
            echo Retrying
            set RETRY=1
        ) else (
            echo Aborting
            call :exit_action
        )
    )
    if %RETRY% EQU 1 call :exec_iap_tool_get_hdmi_version
goto:eof


:: *** get NMR version *********************************************************

:exec_iap_tool_get_nmr_version
    set RETRY=0
    set IAP_NMR_VERSION=-1
    if "%BANNER%" NEQ "" echo %BANNER%
    for /F "delims=" %%i in ('%IAP_TOOL% version') do (
        set temp=%%i
        set temp=!temp:NMR^:=!
        if %%i NEQ !temp! (
            set IAP_NMR_VERSION=!temp!
        )
    )
    if !IAP_NMR_VERSION! NEQ -1 (
        for /F "tokens=1 delims= " %%a in ("!IAP_NMR_VERSION!") do set IAP_NMR_VERSION=%%a
    )
    if !IAP_NMR_VERSION! EQU -1 (
        set RETRY_BANNER=Warning: Process returned an error.
        set RETRY_QUESTION="Would you like to retry (R) or abort (a) ? "
        echo(
        call :ask_user_yes_no R a R
        if !ANSW! EQU 1 (
            echo Retrying
            set RETRY=1
        ) else (
            echo Aborting
            call :exit_action
        )
    )
    if %RETRY% EQU 1 call :exec_iap_tool_get_nmr_version
goto:eof


:: *** test Bootloader mode ****************************************************

:test_bootloader_mode
    %IAP_TOOL% version -b 2> nul
    if %errorlevel% EQU 0 (
        goto:eof
    )
    echo Windows may need to install a driver automatically.
    echo Please be patient, we will wait here up to 5 minutes...
    for /L %%i in (1,1,300) do (
        %IAP_TOOL% version -b 2> nul
        set RET=!errorlevel!
        if !RET! EQU 0 (
            goto:eof
        )
        <nul (set/p _any_variable=.)
        ping -n 2 -w 1000 127.0.0.1 > nul
    )
    echo(
    echo(
    echo Error
    echo(
    echo We were not able to switch to bootloader mode.
    echo If Windows is still installing a driver please wait until it is done.
    echo Then turn the IAP off and restart the update procedure.
    set errorlevel=0
    call :exit_action
goto:eof


:entry_firmware_update
:: *** enable command extensions if necessary **********************************

verify other 2>nul
setlocal enableextensions enabledelayedexpansion
if %errorlevel% NEQ 0 (
    echo Error: Unable to enable command extensions
    call :exit_action
)


:: *** display banner **********************************************************

echo(
echo                            Illusonic Audio Processor
echo                                 Illusonic GmbH
echo(
echo(
echo This tool will update the firmware of the Illusonic Audio Processor.
echo All settings will be preserved. A backup of the settings will be saved in
echo '%IAP_SETTINGS%'.
echo(
echo(
echo Please make sure the Illusonic Audio Processor is connected through USB,
echo power is applied and the power switch on the back is turned on.
echo(

set RETRY_BANNER=
set RETRY_QUESTION="Do you want to continue [y/N] ? "
call :ask_user_yes_no y N N
if !ANSW! NEQ 1 (
    echo Canceled
    call :exit_action
)


:: *** create app support directory ********************************************

if NOT EXIST "%LIBDIR%" mkdir "%LIBDIR%"


:: *** detect current PIC version **********************************************

set V=-1
echo(
echo Detecting PIC firmware version...
%IAP_TOOL% detect
if %errorlevel% EQU 0 (
    :: PIC supported by latest iap2_tool, firmware mode
    set V=0
    set MCU_MODE=0
    goto detect_ok
)
if %errorlevel% EQU -1 (
    goto detect_failed
)
%IAP_TOOL_vA_1_7% detect
if %errorlevel% EQU 0 (
    :: PIC supported by older iap2_tool, firmware mode
    set V=1007
    set MCU_MODE=0
    goto detect_ok
)
%IAP_TOOL_vA_1_6% detect
if %errorlevel% EQU 0 (
    :: PIC supported by older iap2_tool, firmware mode
    set V=1006
    set MCU_MODE=0
    goto detect_ok
)
%IAP_TOOL_vA_1_0% detect
if %errorlevel% EQU 0 (
    :: PIC supported by older iap2_tool, firmware mode
    set V=1000
    set MCU_MODE=0
    goto detect_ok
)
if %errorlevel% EQU 1 (
    :: bootloader mode
    set V=0
    set MCU_MODE=1
    goto detect_ok
)
:detect_failed
echo Error: Illusonic Audio Processor not detected, exiting...
call :exit_action
:detect_ok


:: *** read and save settings **************************************************

if %MCU_MODE% EQU 0 (
    echo(
    echo Reading IAP settings...
    if %V% EQU 0 (
        %IAP_TOOL% config -o "%IAP_SETTINGS_TMP%"
    ) else if %V% EQU 1000 (
        %IAP_TOOL_vA_1_0% config -o "%IAP_SETTINGS_TMP%"
    ) else if %V% EQU 1006 (
        %IAP_TOOL_vA_1_6% config -o "%IAP_SETTINGS_TMP%"
    ) else if %V% EQU 1007 (
        %IAP_TOOL_vA_1_7% config -o "%IAP_SETTINGS_TMP%"
    )
    set RET=%errorlevel%
) else (
    set RET=1
)
if %RET% NEQ 0 (
    echo(
    echo Warning: Unable to save the current settings from the IAP.
    echo Warning: If you continue, your device will be reset to factory defaults.
    echo(
    set RETRY_BANNER=
    set RETRY_QUESTION="Do you want to continue [y/N] ? "
    call :ask_user_yes_no y N N
    if !ANSW! NEQ 1 (
        echo Canceled
        call :exit_action
    )
    set SETTINGS_SAVED=0
) else (
    set SETTINGS_SAVED=1
)


:: *** switch to bootloader, wait for driver installation **********************

:: This is done after saving the settings but before rotating then. On purpose!
echo(
echo Switching to Bootloader mode...
call :test_bootloader_mode


:: *** rotate old settings files ***********************************************

if %SETTINGS_SAVED% EQU 0 goto save_settings_file
if NOT EXIST "%IAP_SETTINGS%" goto save_settings_file

set i=0
:rotate_loop_1
if EXIST "%IAP_SETTINGS%.%i%" (
    if %i% LSS 9 (
        set /A i+=1
        goto rotate_loop_1
    )
)
set /A j=%i%
set /A i-=1
:rotate_loop_2
if %i% GEQ 0 (
    move /Y "%IAP_SETTINGS%.%i%" "%IAP_SETTINGS%.%j%" > nul
    set /A i-=1
    set /A j-=1
    goto rotate_loop_2
)
move /Y "%IAP_SETTINGS%" "%IAP_SETTINGS%.0" > nul


:: *** save new settings file **************************************************

:save_settings_file
if %SETTINGS_SAVED% EQU 1 move /Y "%IAP_SETTINGS_TMP%" "%IAP_SETTINGS%" > nul


:: *** update PIC firmware *****************************************************

echo(
set BANNER=Updating PIC firmware (~ 15 sec) ...
call :exec_iap_tool flash_pic "%PIC_FIRMWARE%"


:: *** detect IAP model ********************************************************

echo(
echo Detecting IAP model...
for /F "tokens=1,2 delims= " %%i in ('%IAP_TOOL% version') do (
    if "%%i"=="MODEL:" (
        set IAP_MODEL=%%j
    )
)
echo %IAP_MODEL%
if not "%IAP_MODEL%"=="IAP02A" (
    if not "%IAP_MODEL%"=="IAP04A" (
        if not "%IAP_MODEL%"=="IAP08A" (
            if not "%IAP_MODEL%"=="IAP16A" (
            	if not "%IAP_MODEL%"=="IAP16A-D" (
					if not "%IAP_MODEL%"=="IAP02A-MK2" (
						if not "%IAP_MODEL%"=="IAP04A-MK2" (
							if not "%IAP_MODEL%"=="IAP08A-MK2" (
								if not "%IAP_MODEL%"=="IAP16A-MK2" (
									if not "%IAP_MODEL%"=="IAP16A-MK2-D" (
										echo Error: this model is not supported %IAP_MODEL%
										set errorlevel=10
										call :exit_action
									)
								)
							)
						)
					)
				)
            )
        )
    )
)


:: *** update DSP firmware *****************************************************

echo(
set BANNER=Updating DSP firmware (~ 17 sec) ...
if "%IAP_MODEL%"=="IAP02A" (
    set DSP_FIRMWARE="%DSP24_FIRMWARE%"
) else if "%IAP_MODEL%"=="IAP04A" (
    set DSP_FIRMWARE="%DSP24_FIRMWARE%"
) else if "%IAP_MODEL%"=="IAP08A" (
    set DSP_FIRMWARE="%DSP08_FIRMWARE%"
) else if "%IAP_MODEL%"=="IAP16A" (
    set DSP_FIRMWARE="%DSP16_FIRMWARE%"
) else if "%IAP_MODEL%"=="IAP16A-D" (
    set DSP_FIRMWARE="%DSP16D_FIRMWARE%"
) else if "%IAP_MODEL%"=="IAP02A-MK2" (
    set DSP_FIRMWARE="%DSP24_FIRMWARE%"
) else if "%IAP_MODEL%"=="IAP04A-MK2" (
    set DSP_FIRMWARE="%DSP24_FIRMWARE%"
) else if "%IAP_MODEL%"=="IAP08A-MK2" (
    set DSP_FIRMWARE="%DSP08_FIRMWARE%"
) else if "%IAP_MODEL%"=="IAP16A-MK2" (
    set DSP_FIRMWARE="%DSP16_FIRMWARE%"
) else if "%IAP_MODEL%"=="IAP16A-MK2-D" (
    set DSP_FIRMWARE="%DSP16D_FIRMWARE%"
) else (
    echo Error: unknown IAP model"
    set errorlevel=10
    call :exit_action
)
call :exec_iap_tool flash_dsp "%DSP_FIRMWARE%"


:: *** restore settings ********************************************************

echo(
if %SETTINGS_SAVED% EQU 1 (
    set BANNER=Restoring IAP settings...
    call :exec_iap_tool config -i "%IAP_SETTINGS%"
) else (
    set BANNER=Restoring factory default settings...
    call :exec_iap_tool factory_default
)
echo(
set BANNER=Saving settings permanently in IAP...
call :exec_iap_tool power off


:: *** power on ****************************************************************

echo(
set BANNER=Booting...
call :exec_iap_tool power on


:: *** update HDMI firmware ****************************************************

echo(
set BANNER=Reading HDMI firmware version...
call :exec_iap_tool_get_hdmi_version

set _prefix=%IAP_HDMI_VERSION:~0,3%
if "%IAP_HDMI_VERSION%"=="%HDMI_VERSION_14%" (
    echo HDMI firmware already up-to-date ^(version: %IAP_HDMI_VERSION%^)
) else if "%IAP_HDMI_VERSION%"=="%HDMI_VERSION_20%" (
    echo HDMI firmware already up-to-date ^(version: %IAP_HDMI_VERSION%^)
) else if "%IAP_HDMI_VERSION%"=="%HDMI_VERSION_21%" (
    echo HDMI firmware already up-to-date ^(version: %IAP_HDMI_VERSION%^)
) else if "%_prefix%"=="%HDMI_VERSION_14_PREFIX%" (
    echo %IAP_HDMI_VERSION%
    echo(
    set BANNER=Updating HDMI firmware ^(~ 2 min^) ...
    call :exec_iap_tool flash_hdmi "%HDMI_FIRMWARE_14%"
) else if "%_prefix%"=="%HDMI_VERSION_20_PREFIX%" (
    echo %IAP_HDMI_VERSION%
    echo(
    set BANNER=Updating HDMI firmware ^(~ 4 min^) ...
    call :exec_iap_tool flash_hdmi "%HDMI_FIRMWARE_20a%"
    call :exec_iap_tool flash_hdcp "%HDMI_FIRMWARE_20b%" "%HDMI_FIRMWARE_20c%"
) else if "%_prefix%"=="%HDMI_VERSION_21_PREFIX%" (
    echo %IAP_HDMI_VERSION%
    echo(
    set BANNER=Updating HDMI firmware ^(~ 4 min^) ...
    call :exec_iap_tool flash_hdcp "%HDMI_FIRMWARE_21b%"
    call :exec_iap_tool flash_hdmi "%HDMI_FIRMWARE_21a%"
) else (
    echo "Error: could not determine the version of the HDMI board".
    echo "       The update will continue but the HDMI board will not be updated."
    echo "       Please contact Illusonic for assistance."
    echo ""
    set errorlevel=10
    call :exit_action
)


:: *** update NMR firmware *****************************************************

echo(
echo Reading NMR firmware version...
for /L %%i in (1,1,60) do (
    set BANNER=
    call :exec_iap_tool_get_nmr_version
    if "!IAP_NMR_VERSION!"=="0.0" (
        <nul (set/p _any_variable=.)
        ping -n 2 -w 1000 127.0.0.1 > nul
    ) else (
        goto nmr_version_ok
    )
)

:nmr_version_ok
if "%NMR_VERSION%"=="%IAP_NMR_VERSION%" (
    echo(
    echo NMR firmware already up-to-date ^(version: %IAP_NMR_VERSION%^)
) else (
    echo %IAP_NMR_VERSION%
    echo(
    set BANNER=Updating NMR firmware ^(~ 6 min^) ...
    call :exec_iap_tool flash_nmr "%NMR_FIRMWARE%"
)


:: *** power off ***************************************************************

echo(
set BANNER=Powering down...
call :exec_iap_tool power off


:: *** DONE ********************************************************************

echo(
echo(
echo ------------------------------------------------------------------------------
echo SUCCESS
echo ------------------------------------------------------------------------------

call :exit_action
